@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2022 Vaclav Slavik
+ *  Copyright (C) 1999-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -27,6 +27,7 @@
 
 #include "catalog_po.h"
 #include "catalog_xliff.h"
+#include "catalog_json.h"
 
 #include "configuration.h"
 #include "errors.h"
@@ -75,6 +76,27 @@ wxString FixBrokenSearchPathValue(wxString p)
     return p;
 }
 
+// Detect whether source strings are just IDs instead of actual text
+bool DetectUseOfSymbolicIDs(Catalog& cat)
+{
+    // Employ a simple heuristic: IDs won't contain whitespace.
+    // This is not enough as is, because some (notably Asian) languages don't use
+    // whitespace, so also check for use of ASCII characters only. Typical non-symbolic
+    // files will fail at least one of the tests in most of their strings.
+    //
+    for (auto& i: cat.items())
+    {
+        for (auto c: i->GetRawString())
+        {
+            if (c == ' ' || c >= 0x80)
+                return false;
+        }
+    }
+
+    wxLogTrace("poedit", "detected use of symbolic IDs for source language");
+    return true;
+}
+
 } // anonymous namespace
 
 
@@ -476,10 +498,8 @@ Catalog::HeaderData::Find(const wxString& key) const
 
 Catalog::Catalog(Type type)
 {
-    m_sourceLanguage = Language::English();
     m_fileType = type;
 
-    m_isOk = true;
     m_header.BasePath = wxEmptyString;
     for(int i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
     {
@@ -488,19 +508,6 @@ Catalog::Catalog(Type type)
 }
 
 
-bool Catalog::HasCapability(Catalog::Cap cap) const
-{
-    switch (cap)
-    {
-        case Cap::Translations:
-        case Cap::LanguageSetting:
-        case Cap::UserComments:
-            return m_fileType == Type::PO;
-    }
-    return false; // silence VC++ warning
-}
-
-
 static inline wxString GetCurrentTimeString()
 {
     return wxDateTime::Now().Format("%Y-%m-%d %H:%M%z");
@@ -544,6 +551,8 @@ void Catalog::CreateNewHeader(const Catalog::HeaderData& pot_header)
     dt.Lang = Language();
     if (dt.LanguageTeam == "LANGUAGE <LL@li.org>")
         dt.LanguageTeam.clear();
+    if (dt.GetHeader("Plural-Forms") == "nplurals=INTEGER; plural=EXPRESSION;")
+        dt.DeleteHeader("Plural-Forms");
 
     // translator should be pre-filled & not the default "FULL NAME <EMAIL@ADDRESS>"
     dt.DeleteHeader("Last-Translator");
@@ -610,12 +619,24 @@ wxString MaskForType(Catalog::Type t)
             return MaskForType("*.pot", _("POT Translation Templates"));
         case Catalog::Type::XLIFF:
             return MaskForType("*.xlf;*.xliff", _("XLIFF Translation Files"));
+        case Catalog::Type::JSON:
+            return MaskForType("*.json", _("JSON Translation Files"));
+        case Catalog::Type::JSON_FLUTTER:
+            // TRANSLATORS: "Flutter" is proper noun, name of a developer tool
+            return MaskForType("*.arb", _("Flutter Translation Files"));
     }
     return ""; // silence stupid warning
 }
 
 } // anonymous namespace
 
+wxString Catalog::GetAllTypesFileMask()
+{
+    return MaskForType("*.po;*.pot;*.xlf;*.xliff;*.json;*.arb", _("All Translation Files"), /*showExt=*/false) +
+        "|" +
+        GetTypesFileMask({ Type::PO, Type::POT, Type::XLIFF, Type::JSON, Type::JSON_FLUTTER });
+}
+
 wxString Catalog::GetTypesFileMask(std::initializer_list<Type> types)
 {
     if (types.size() == 0)
@@ -631,13 +652,6 @@ wxString Catalog::GetTypesFileMask(std::initializer_list<Type> types)
     return out;
 }
 
-wxString Catalog::GetAllTypesFileMask()
-{
-    return MaskForType("*.po;*.pot;*.xlf;*.xliff", _("All Translation Files"), /*showExt=*/false) +
-           "|" +
-           GetTypesFileMask({Type::PO, Type::POT, Type::XLIFF});
-}
-
 
 void Catalog::SetFileName(const wxString& fn)
 {
@@ -757,9 +771,6 @@ bool Catalog::HasSourcesAvailable() const
 
 std::shared_ptr<SourceCodeSpec> Catalog::GetSourceCodeSpec() const
 {
-    if (!m_isOk)
-        return nullptr;
-
     auto path = GetSourcesBasePath();
     if (!path.empty())
     {
@@ -788,33 +799,9 @@ std::shared_ptr<SourceCodeSpec> Catalog::GetSourceCodeSpec() const
 }
 
 
-static unsigned GetCountFromPluralFormsHeader(const Catalog::HeaderData& header)
-{
-    if ( header.HasHeader("Plural-Forms") )
-    {
-        // e.g. "Plural-Forms: nplurals=3; plural=(n%10==1 && n%100!=11 ?
-        //       0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);\n"
-
-        wxString form = header.GetHeader("Plural-Forms");
-        form = form.BeforeFirst(_T(';'));
-        if (form.BeforeFirst(_T('=')) == "nplurals")
-        {
-            wxString vals = form.AfterFirst('=');
-            if (vals == "INTEGER") // POT default
-                return 2;
-            long val;
-            if (vals.ToLong(&val))
-                return (unsigned)val;
-        }
-    }
-
-    // fallback value for plural forms count should be 2, as in English:
-    return 2;
-}
-
 unsigned Catalog::GetPluralFormsCount() const
 {
-    unsigned count = GetCountFromPluralFormsHeader(m_header);
+    unsigned count = 0;
 
     for (auto& i: m_items)
     {
@@ -824,40 +811,10 @@ unsigned Catalog::GetPluralFormsCount() const
     return count;
 }
 
-bool Catalog::HasWrongPluralFormsCount() const
-{
-    unsigned count = 0;
-
-    for (auto& i: m_items)
-    {
-        count = std::max(count, i->GetPluralFormsCount());
-    }
-
-    if ( count == 0 )
-        return false; // nothing translated, so we can't tell
-
-    // if 'count' is less than the count from header, it may simply mean there
-    // are untranslated strings
-    if ( count > GetCountFromPluralFormsHeader(m_header) )
-        return true;
-
-    return false;
-}
-
-bool Catalog::HasPluralItems() const
-{
-    for (auto& i: m_items)
-    {
-        if ( i->HasPlural() )
-            return true;
-    }
-    return false;
-}
-
 void Catalog::SetLanguage(Language lang)
 {
+    // FIXME: move m_header to POCatalog too
     m_header.Lang = lang;
-    m_header.SetHeaderNotEmpty("Plural-Forms", lang.DefaultPluralFormsExpr().str());
 }
 
 void Catalog::GetStatistics(int *all, int *fuzzy, int *badtokens,
@@ -937,6 +894,9 @@ wxString CatalogItem::GetFlags() const
 
 wxString CatalogItem::GetFormatFlag() const
 {
+    if (m_moreFlags.empty())
+        return wxString();
+
     auto pos = m_moreFlags.find(wxS("-format"));
     if (pos == wxString::npos)
         return wxString();
@@ -1081,6 +1041,93 @@ wxString CatalogItem::GetOldMsgid() const
 }
 
 
+Catalog::ValidationResults Catalog::Validate(const wxString& /*fileWithSameContent*/)
+{
+    ValidationResults res;
+
+    for (auto& i: m_items)
+        i->ClearIssue();
+    res.errors = 0;
+
+    if (!HasCapability(Catalog::Cap::Translations))
+        return res; // no errors in POT files
+
+#if wxUSE_GUI
+    if (Config::ShowWarnings())
+    {
+        // TODO: _some_ checks (e.g. plurals) do make sense even with symbolic IDs
+        if (!UsesSymbolicIDsForSource())
+            res.warnings = QAChecker::GetFor(*this)->Check(*this);
+    }
+#endif
+
+    return res;
+}
+
+
+void Catalog::PostCreation()
+{
+    if (!m_sourceLanguage.IsValid())
+    {
+        if (!m_sourceIsSymbolicID)
+            m_sourceIsSymbolicID = DetectUseOfSymbolicIDs(*this);
+
+        if (!m_sourceIsSymbolicID)
+        {
+            // detect source language from the text (ignoring plurals for simplicity,
+            // as we don't need 100% of the text):
+            wxString allText;
+            for (auto& i: items())
+            {
+                allText.append(i->GetRawString());
+                allText.append('\n');
+            }
+            if (!allText.empty())
+            {
+                m_sourceLanguage = Language::TryDetectFromText(allText.utf8_str());
+                wxLogTrace("poedit", "detected source language is '%s'", m_sourceLanguage.Code());
+            }
+        }
+    }
+
+    // All the following fixups are for files that contain translations (i.e. not POTs)
+    if (!HasCapability(Cap::Translations))
+        return;
+
+    if (!GetLanguage().IsValid())
+    {
+        Language lang;
+        if (!m_fileName.empty())
+        {
+            lang = Language::TryGuessFromFilename(m_fileName);
+            wxLogTrace("poedit", "guessed translation language from filename '%s' is '%s'", m_fileName, lang.Code());
+        }
+
+        if (!lang.IsValid())
+        {
+            // If all else fails, try to detect the language from content
+            wxString allText;
+            for (auto& i: items())
+            {
+                if (!i->IsTranslated())
+                    continue;
+                allText.append(i->GetTranslation());
+                allText.append('\n');
+            }
+            if (!allText.empty())
+            {
+                lang = Language::TryDetectFromText(allText.utf8_str());
+                wxLogTrace("poedit", "detected translation language is '%s'", GetLanguage().Code());
+            }
+        }
+
+        if (lang.IsValid())
+            SetLanguage(lang);
+    }
+}
+
+
+
 // Catalog file creation factories:
 
 CatalogPtr Catalog::Create(Type type)
@@ -1089,10 +1136,12 @@ CatalogPtr Catalog::Create(Type type)
     {
         case Type::PO:
         case Type::POT:
-            return std::make_shared<POCatalog>(type);
+            return CatalogPtr(new POCatalog(type));
 
         case Type::XLIFF:
-            wxFAIL_MSG("empty XLIFF creation not implemented");
+        case Type::JSON:
+        case Type::JSON_FLUTTER:
+            wxFAIL_MSG("empty XLIFF/JSON creation not implemented");
             return CatalogPtr();
     }
 
@@ -1108,16 +1157,20 @@ CatalogPtr Catalog::Create(const wxString& filename, int flags)
     CatalogPtr cat;
     if (POCatalog::CanLoadFile(ext))
     {
-        cat = std::make_shared<POCatalog>(filename, flags);
+        cat.reset(new POCatalog(filename, flags));
         flags = 0; // don't do the stuff below that is already handled by POCatalog's parser
     }
     else if (XLIFFCatalog::CanLoadFile(ext))
     {
         cat = XLIFFCatalog::Open(filename);
     }
+    else if (JSONCatalog::CanLoadFile(ext))
+    {
+        cat = JSONCatalog::Open(filename);
+    }
 
     if (!cat)
-        throw Exception(wxString::Format(_(L"File “%s” is in unsupported format."), filename));
+        throw Exception(_("The file is in a format not recognized by Poedit."));
 
     if (flags & CreationFlag_IgnoreTranslations)
     {
@@ -1125,6 +1178,9 @@ CatalogPtr Catalog::Create(const wxString& filename, int flags)
             item->ClearTranslation();
     }
 
+    cat->SetFileName(filename);
+    cat->PostCreation();
+
     return cat;
 }
 
@@ -1133,5 +1189,43 @@ bool Catalog::CanLoadFile(const wxString& extension_)
     auto extension = extension_.Lower();
 
     return POCatalog::CanLoadFile(extension) ||
-           XLIFFCatalog::CanLoadFile(extension);
+           XLIFFCatalog::CanLoadFile(extension) ||
+           JSONCatalog::CanLoadFile(extension);
+}
+
+
+void Catalog::SideloadSourceDataFromReferenceFile(CatalogPtr ref)
+{
+    std::map<wxString, CatalogItemPtr> refItems;
+
+    for (auto iref: ref->items())
+        refItems[iref->GetRawString()] = iref;
+
+    for (auto i: this->items())
+    {
+        auto ri = refItems.find(i->GetRawString());
+        if (ri == refItems.end())
+            continue;
+
+        auto& rdata = *ri->second;
+        auto d = std::make_shared<SideloadedItemData>();
+        d->source_string = rdata.GetTranslation();
+        if (rdata.HasPlural())
+            d->source_plural_string = rdata.GetTranslation(1);
+        if (rdata.HasExtractedComments())
+            d->extracted_comments = rdata.GetExtractedComments();
+
+        i->AttachSideloadedData(d);
+    }
+
+    m_sideloaded = std::make_shared<SideloadedCatalogData>();
+    m_sideloaded->reference_file = ref;
+    m_sideloaded->source_language = ref->GetLanguage();
+}
+
+void Catalog::ClearSideloadedSourceData()
+{
+    m_sideloaded.reset();
+    for (auto i: this->items())
+        i->ClearSideloadedData();
 }
