@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2022 Vaclav Slavik
+ *  Copyright (C) 1999-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,6 @@
 #include "errors.h"
 #include "extractors/extractor.h"
 #include "gexecute.h"
-#include "qa_checks.h"
 #include "str_helpers.h"
 #include "utility.h"
 #include "version.h"
@@ -185,6 +184,31 @@ wxTextFileType GetDesiredCRLFFormat(wxTextFileType existingCRLF)
     }
 }
 
+
+unsigned GetCountFromPluralFormsHeader(const Catalog::HeaderData& header)
+{
+    if ( header.HasHeader("Plural-Forms") )
+    {
+        // e.g. "Plural-Forms: nplurals=3; plural=(n%10==1 && n%100!=11 ?
+        //       0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2);\n"
+
+        wxString form = header.GetHeader("Plural-Forms");
+        form = form.BeforeFirst(_T(';'));
+        if (form.BeforeFirst(_T('=')) == "nplurals")
+        {
+            wxString vals = form.AfterFirst('=');
+            if (vals == "INTEGER") // POT default
+                return 2;
+            long val;
+            if (vals.ToLong(&val))
+                return (unsigned)val;
+        }
+    }
+
+    // fallback value for plural forms count should be 2, as in English:
+    return 2;
+}
+
 } // anonymous namespace
 
 
@@ -600,25 +624,11 @@ class POLoadParser : public POCatalogParser
         POLoadParser(POCatalog& c, wxTextFile *f)
               : POCatalogParser(f),
                 FileIsValid(false),
-                m_catalog(c), m_nextId(1), m_seenHeaderAlready(false), m_collectMsgidText(true) {}
+                m_catalog(c), m_nextId(1), m_seenHeaderAlready(false) {}
 
         // true if the file is valid, i.e. has at least some data
         bool FileIsValid;
 
-        Language GetMsgidLanguage()
-        {
-            auto lang = GetSpecifiedMsgidLanguage();
-            if (lang.IsValid())
-                return lang;
-
-            auto utf8 = m_allMsgidText.utf8_str();
-            lang = Language::TryDetectFromText(utf8.data(), utf8.length());
-            if (!lang.IsValid())
-                lang = Language::English();  // gettext historically assumes English
-            return lang;
-        }
-
-    protected:
         Language GetSpecifiedMsgidLanguage()
         {
             auto x_srclang = m_catalog.Header().GetHeader("X-Source-Language");
@@ -633,6 +643,7 @@ class POLoadParser : public POCatalogParser
             return Language();
         }
 
+    protected:
         POCatalog& m_catalog;
 
         virtual bool OnEntry(const wxString& msgid,
@@ -660,10 +671,6 @@ class POLoadParser : public POCatalogParser
     private:
         int m_nextId;
         bool m_seenHeaderAlready;
-
-        // collected text of msgids, with newlines, for language detection
-        bool m_collectMsgidText;
-        wxString m_allMsgidText;
 };
 
 
@@ -691,7 +698,6 @@ bool POLoadParser::OnEntry(const wxString& msgid,
             // gettext header:
             m_catalog.m_header.FromString(mtranslations[0]);
             m_catalog.m_header.Comment = comment;
-            m_collectMsgidText = !GetSpecifiedMsgidLanguage().IsValid();
             m_seenHeaderAlready = true;
         }
         // else: ignore duplicate header in malformed files
@@ -704,7 +710,10 @@ bool POLoadParser::OnEntry(const wxString& msgid,
             d->SetFlags(flags);
         d->SetString(msgid);
         if (has_plural)
+        {
+            m_catalog.m_hasPluralItems = true;
             d->SetPluralString(msgid_plural);
+        }
         if (has_context)
             d->SetContext(context);
         d->SetTranslations(mtranslations);
@@ -724,18 +733,6 @@ bool POLoadParser::OnEntry(const wxString& msgid,
         }
         d->SetOldMsgid(msgid_old);
         m_catalog.AddItem(d);
-
-        // collect text for language detection:
-        if (m_collectMsgidText)
-        {
-            m_allMsgidText.append(msgid);
-            m_allMsgidText.append('\n');
-            if (!msgid_plural.empty())
-            {
-                m_allMsgidText.append(msgid_plural);
-                m_allMsgidText.append('\n');
-            }
-        }
     }
     return true;
 }
@@ -769,25 +766,52 @@ bool POLoadParser::OnDeletedEntry(const wxArrayString& deletedLines,
 wxArrayString POCatalogItem::GetReferences() const
 {
     // A line may contain several references, separated by white-space.
-    // Each reference is in the form "path_name:line_number"
-    // (path_name may contain spaces)
-
+    // Traditionally, each reference was in the form "path_name:line_number", but non
+    // standard references are sometime used too, including hyperlinks.
+    // Filenames that contain spaces are supported - they must be enclosed by Unicode
+    // characters U+2068 and U+2069.
     wxArrayString refs;
 
-    for ( wxArrayString::const_iterator i = m_references.begin(); i != m_references.end(); ++i )
+    for (auto ref = m_references.begin(); ref != m_references.end(); ++ref)
     {
-        wxString line = *i;
+        auto line = ref->Strip(wxString::both);
+        wxString buf;
 
-        line = line.Strip(wxString::both);
-        while (!line.empty())
+        auto i = line.begin();
+        while (i != line.end())
         {
-            size_t idx = 0;
-            while (idx < line.length() && line[idx] != _T(':')) { idx++; }
-            while (idx < line.length() && !wxIsspace(line[idx])) { idx++; }
-
-            refs.push_back(line.Left(idx));
-            line = line.Mid(idx).Strip(wxString::both);
+            const wchar_t c = *i;
+            if (wxIsspace(c))
+            {
+                // store reference text encountered so far:
+                if (!buf.empty())
+                {
+                    refs.push_back(buf);
+                    buf.clear();
+                }
+                ++i;
+            }
+            else if (c == L'\u2068')
+            {
+                // quoted filename between U+2068 and U+2069:
+                ++i;
+                while (i != line.end() && *i != L'\u2069')
+                {
+                    buf += *i;
+                    ++i;
+                }
+                if (i != line.end())
+                    ++i; // skip trailing U+2069
+            }
+            else
+            {
+                buf += c;
+                ++i;
+            }
         }
+
+        if (!buf.empty())
+            refs.push_back(buf);
     }
 
     return refs;
@@ -809,7 +833,16 @@ POCatalog::POCatalog(const wxString& po_file, int flags) : Catalog(Type::PO)
     m_fileCRLF = wxTextFileType_None;
     m_fileWrappingWidth = DEFAULT_WRAPPING;
 
-    m_isOk = Load(po_file, flags);
+    Load(po_file, flags);
+}
+
+void POCatalog::PostCreation()
+{
+    Catalog::PostCreation();
+    
+    // gettext historically assumes English:
+    if (!m_sourceLanguage.IsValid() && !m_sourceIsSymbolicID)
+        m_sourceLanguage = Language::English();
 }
 
 
@@ -820,6 +853,7 @@ bool POCatalog::HasCapability(Catalog::Cap cap) const
         case Cap::Translations:
         case Cap::LanguageSetting:
         case Cap::UserComments:
+        case Cap::FuzzyTranslations:
             return m_fileType == Type::PO;
     }
     return false; // silence VC++ warning
@@ -841,11 +875,12 @@ wxString POCatalog::GetPreferredExtension() const
         case Type::POT:
             return "pot";
 
-        case Type::XLIFF:
-            wxFAIL_MSG("not possible here"); 
+        default:
+            wxFAIL_MSG("not possible here");
+            return "po";
     }
 
-    return "";
+    return "po";
 }
 
 
@@ -855,12 +890,11 @@ static inline wxString GetCurrentTimeString()
 }
 
 
-bool POCatalog::Load(const wxString& po_file, int flags)
+void POCatalog::Load(const wxString& po_file, int flags)
 {
     wxTextFile f;
 
     Clear();
-    m_isOk = false;
     m_fileName = po_file;
     m_header.BasePath = wxEmptyString;
 
@@ -874,7 +908,9 @@ bool POCatalog::Load(const wxString& po_file, int flags)
     /* Load the .po file: */
 
     if (!f.Open(po_file, wxConvISO8859_1))
-        return false;
+    {
+        throw Exception(_(L"Couldn’t load the file, it is probably damaged."));
+    }
 
     {
         wxLogNull null; // don't report parsing errors from here, report them later
@@ -886,7 +922,9 @@ bool POCatalog::Load(const wxString& po_file, int flags)
     f.Close();
     wxCSConv encConv(m_header.Charset);
     if (!f.Open(po_file, encConv))
-        return false;
+    {
+        throw Exception(_(L"Couldn’t load the file, it is probably damaged."));
+    }
 
     if (!VerifyFileCharset(f, po_file, m_header.Charset))
     {
@@ -898,14 +936,10 @@ bool POCatalog::Load(const wxString& po_file, int flags)
     parser.IgnoreTranslations(flags & CreationFlag_IgnoreTranslations);
     if (!parser.Parse())
     {
-        wxLogError(
-            wxString::Format(
-                _(L"Couldn’t load file %s, it is probably corrupted."),
-                po_file.c_str()));
-        return false;
+        throw Exception(_(L"Couldn’t load the file, it is probably damaged."));
     }
 
-    m_sourceLanguage = parser.GetMsgidLanguage();
+    m_sourceLanguage = parser.GetSpecifiedMsgidLanguage();  // may be, and likely will, invalid
 
     // now that the catalog is loaded, update its items with the bookmarks
     for (unsigned i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
@@ -930,9 +964,9 @@ bool POCatalog::Load(const wxString& po_file, int flags)
 
     // If we didn't find any entries, the file must be invalid:
     if (!parser.FileIsValid)
-        return false;
-
-    m_isOk = true;
+    {
+        throw Exception(_(L"Couldn’t load the file, it is probably damaged."));
+    }
 
     f.Close();
 
@@ -940,8 +974,6 @@ bool POCatalog::Load(const wxString& po_file, int flags)
 
     if ( flags & CreationFlag_IgnoreHeader )
         CreateNewHeader();
-
-    return true;
 }
 
 
@@ -950,41 +982,25 @@ void POCatalog::FixupCommonIssues()
     if (m_header.Project == "PACKAGE VERSION")
         m_header.Project.clear();
 
-    // All the following fixups are specific to POs and should *not* be done in POTs:
-    if (m_fileType == Type::POT)
-        return;
-
-    if (!m_header.Lang.IsValid())
+    // In PHP use, strings with % (typically: 100%) get frequently mis-identified as php-format, because the
+    // format string allows space, so e.g. "100% complete" has a valid "% c" format flag in it. Work around
+    // this by removing the flag ourselves, as translators can rarely influence it:
+    for (auto& i: items())
     {
-        if (!m_fileName.empty())
+        if (i->GetFormatFlag() == wxS("php"))
         {
-            m_header.Lang = Language::TryGuessFromFilename(m_fileName);
-            wxLogTrace("poedit", "guessed language from filename '%s': %s", m_fileName, m_header.Lang.Code());
-        }
-
-        if (!m_header.Lang.IsValid())
-        {
-            // If all else fails, try to detect the language from content
-            wxString allText;
-            for (auto& i: items())
+            auto s = i->GetRawString();
+            if (s.Contains(wxS("% ")) && !s.Contains(wxS("%% ")))
             {
-                for (auto& s: i->GetTranslations())
-                {
-                    if (s.empty())
-                        continue;
-                    allText.append(s);
-                    allText.append('\n');
-                }
-            }
-            if (!allText.empty())
-            {
-                auto utf8 = allText.utf8_str();
-                m_header.Lang = Language::TryDetectFromText(utf8.data(), utf8.length());
+                auto poi = std::dynamic_pointer_cast<POCatalogItem>(i);
+                poi->m_moreFlags.Replace("php-format", "no-php-format");
             }
         }
     }
 
-    wxLogTrace("poedit", "translation language is '%s'", GetLanguage().Code());
+    // All the following fixups are specific to POs and should *not* be done in POTs:
+    if (m_fileType == Type::POT)
+        return;
 
     if (m_header.GetHeader("Language-Team") == "LANGUAGE <LL@li.org>")
     {
@@ -1022,8 +1038,6 @@ void POCatalog::FixupCommonIssues()
                 m_header.SetHeader("Plural-Forms", pluralForms);
         }
     }
-
-    // TODO: mark catalog as modified if any changes were made
 }
 
 
@@ -1031,7 +1045,6 @@ void POCatalog::Clear()
 {
     // Catalog base class fields:
     m_items.clear();
-    m_isOk = true;
     for (int i = BOOKMARK_0; i < BOOKMARK_LAST; i++)
         m_header.Bookmarks[i] = -1;
 
@@ -1168,7 +1181,7 @@ bool POCatalog::Save(const wxString& po_file, bool save_mo,
                 m_header.CreationDate = currentTime;
             break;
 
-        case Type::XLIFF:
+        default:
             wxFAIL_MSG("not possible here");
             break;
     }
@@ -1189,7 +1202,7 @@ bool POCatalog::Save(const wxString& po_file, bool save_mo,
 
     try
     {
-        validation_results = DoValidate(po_file_temp);
+        validation_results = Validate(/*fileWithSameContent=*/po_file_temp);
     }
     catch (...)
     {
@@ -1367,7 +1380,7 @@ bool POCatalog::Save(const wxString& po_file, bool save_mo,
         }
     }
 
-    m_fileName = po_file;
+    SetFileName(po_file);
 
     return true;
 }
@@ -1419,7 +1432,7 @@ bool POCatalog::CompileToMO(const wxString& mo_file,
         return false;
     }
 
-    validation_results = DoValidate(po_file_temp);
+    validation_results = Validate(/*fileWithSameContent=*/po_file_temp);
 
     TempOutputFileFor mo_file_temp_obj(mo_file);
     const wxString mo_file_temp = mo_file_temp_obj.FileName();
@@ -1510,11 +1523,11 @@ bool POCatalog::DoSaveOnly(wxTextBuffer& f, wxTextFileType crlf)
         {
             SaveMultiLines(f, wxS("msgctxt \"") + FormatStringForFile(data->GetContext()) + wxS("\""));
         }
-        dummy = FormatStringForFile(data->GetString());
+        dummy = FormatStringForFile(data->GetRawString());
         SaveMultiLines(f, wxS("msgid \"") + dummy + wxS("\""));
         if (data->HasPlural())
         {
-            dummy = FormatStringForFile(data->GetPluralString());
+            dummy = FormatStringForFile(data->GetRawPluralString());
             SaveMultiLines(f, wxS("msgid_plural \"") + dummy + wxS("\""));
 
             for (unsigned i = 0; i < pluralsCount; i++)
@@ -1573,6 +1586,39 @@ bool POCatalog::DoSaveOnly(wxTextBuffer& f, wxTextFileType crlf)
     return f.Write(crlf, wxCSConv(m_header.Charset));
 }
 
+void POCatalog::SetLanguage(Language lang)
+{
+    Catalog::SetLanguage(lang);
+    // don't add unneeded header, but always update it if already present:
+    if (HasPluralItems() || m_header.HasHeader("Plural-Forms"))
+        m_header.SetHeaderNotEmpty("Plural-Forms", lang.DefaultPluralFormsExpr().str());
+}
+
+unsigned POCatalog::GetPluralFormsCount() const
+{
+    return std::max(GetCountFromPluralFormsHeader(m_header), Catalog::GetPluralFormsCount());
+}
+
+bool POCatalog::HasWrongPluralFormsCount() const
+{
+    unsigned count = 0;
+
+    for (auto& i: m_items)
+    {
+        count = std::max(count, i->GetPluralFormsCount());
+    }
+
+    if ( count == 0 )
+        return false; // nothing translated, so we can't tell
+
+    // if 'count' is less than the count from header, it may simply mean there
+    // are untranslated strings
+    if ( count > GetCountFromPluralFormsHeader(m_header) )
+        return true;
+
+    return false;
+}
+
 
 bool POCatalog::HasDuplicateItems() const
 {
@@ -1580,7 +1626,7 @@ bool POCatalog::HasDuplicateItems() const
     std::set<MsgId> ids;
     for (auto& item: m_items)
     {
-        if (!ids.emplace(std::make_pair(item->GetContext(), item->GetString())).second)
+        if (!ids.emplace(std::make_pair(item->GetContext(), item->GetRawString())).second)
             return true;
     }
     return false;
@@ -1612,39 +1658,43 @@ bool POCatalog::FixDuplicateItems()
     if (!wxFileName::FileExists(po_file_fixed))
         return false;
 
-    bool ok = Load(po_file_fixed);
+    Load(po_file_fixed);
     m_fileName = oldname;
-    return ok;
+    PostCreation();
+
+    return true;
 }
 
 
-Catalog::ValidationResults POCatalog::Validate(bool wasJustLoaded)
+Catalog::ValidationResults POCatalog::Validate(const wxString& fileWithSameContent)
 {
-    if (!HasCapability(Catalog::Cap::Translations))
-        return ValidationResults();  // no errors in POT files
+    ValidationResults res = Catalog::Validate(fileWithSameContent);
 
-    if (wasJustLoaded)
+    if (!HasCapability(Catalog::Cap::Translations))
+        return res;  // no errors in POT files
+
+    if (!fileWithSameContent.empty())
     {
-        return DoValidate(GetFileName());
+        ValidateWithMsgfmt(res, fileWithSameContent);
     }
     else
     {
         TempDirectory tmpdir;
         if ( !tmpdir.IsOk() )
-            return ValidationResults();
+            return res;
 
         wxString tmp_po = tmpdir.CreateFileName("validated.po");
         if ( !DoSaveOnly(tmp_po, wxTextFileType_Unix) )
-            return ValidationResults();
+            return res;
 
-        return DoValidate(tmp_po);
+        ValidateWithMsgfmt(res, tmp_po);
     }
+
+    return res;
 }
 
-Catalog::ValidationResults POCatalog::DoValidate(const wxString& po_file)
+void POCatalog::ValidateWithMsgfmt(Catalog::ValidationResults& res, const wxString& po_file)
 {
-    ValidationResults res;
-
     GettextErrors err;
     ExecuteGettextAndParseOutput
     (
@@ -1652,15 +1702,7 @@ Catalog::ValidationResults POCatalog::DoValidate(const wxString& po_file)
         err
     );
 
-    for (auto& i: m_items)
-        i->ClearIssue();
-
-    res.errors = (int)err.size();
-
-#if wxUSE_GUI
-    if (Config::ShowWarnings())
-        res.warnings = QAChecker::GetFor(*this)->Check(*this);
-#endif
+    res.errors += (int)err.size();
 
     for ( GettextErrors::const_iterator i = err.begin(); i != err.end(); ++i )
     {
@@ -1676,21 +1718,21 @@ Catalog::ValidationResults POCatalog::DoValidate(const wxString& po_file)
         // if not matched to an item:
         wxLogError(i->text);
     }
-
-    return res;
 }
 
 
 bool POCatalog::UpdateFromPOT(const wxString& pot_file, bool replace_header)
 {
-    POCatalogPtr pot = std::make_shared<POCatalog>(pot_file, CreationFlag_IgnoreTranslations);
-    if (!pot->IsOk())
+    try
+    {
+        POCatalogPtr pot = std::dynamic_pointer_cast<POCatalog>(Catalog::Create(pot_file, CreationFlag_IgnoreTranslations));
+        return UpdateFromPOT(pot, replace_header);
+    }
+    catch (...) // FIXME
     {
         wxLogError(_(L"“%s” is not a valid POT file."), pot_file.c_str());
         return false;
     }
-
-    return UpdateFromPOT(pot, replace_header);
 }
 
 bool POCatalog::UpdateFromPOT(POCatalogPtr pot, bool replace_header)
@@ -1706,10 +1748,13 @@ bool POCatalog::UpdateFromPOT(POCatalogPtr pot, bool replace_header)
         case Type::POT:
         {
             m_items = pot->m_items;
+            m_sourceLanguage = pot->m_sourceLanguage;
+            m_sourceIsSymbolicID = pot->m_sourceIsSymbolicID;
+            m_hasPluralItems = pot->m_hasPluralItems;
             break;
         }
 
-        case Type::XLIFF:
+        default:
             wxFAIL_MSG("not possible here");
             break;
     }
@@ -1722,7 +1767,7 @@ bool POCatalog::UpdateFromPOT(POCatalogPtr pot, bool replace_header)
 
 POCatalogPtr POCatalog::CreateFromPOT(POCatalogPtr pot)
 {
-    POCatalogPtr c = std::make_shared<POCatalog>();
+    POCatalogPtr c(new POCatalog(Type::PO));
     if (c->UpdateFromPOT(pot, /*replace_header=*/true))
         return c;
     else
@@ -1767,6 +1812,8 @@ bool POCatalog::Merge(const POCatalogPtr& refcat)
         const wxString charset = m_header.Charset;
 
         Load(tmp3);
+        m_fileName = oldname;
+        PostCreation();
 
         // msgmerge doesn't always preserve the charset, it tends to pick
         // the most generic one of the charsets used, so if we are merging with
