@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2017-2022 Vaclav Slavik
+ *  Copyright (C) 2017-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -90,7 +90,7 @@ public:
         return std::make_shared<Dest>(name, std::move(func));
     }
 
-    static const wxString& GetCacheDir()
+    static wxString GetCacheDir(const wxString& category)
     {
         static wxString localBaseDir;
 
@@ -107,7 +107,7 @@ public:
         #endif
         }
 
-        return localBaseDir;
+        return localBaseDir + wxFILE_SEP_PATH + category;
     }
 };
 
@@ -134,7 +134,7 @@ public:
         : CloudSyncProgressWindow(parent)
     {
         // TRANSLATORS: %s is a cloud destination, e.g. "Crowdin" or ftp.wordpress.com etc.
-        Activity->Start(wxString::Format(_(L"Syncing with %s…"), dest->GetName()));
+        Activity->Start(wxString::Format(_(L"Uploading translations to %s…"), dest->GetName()));
     }
 
     ActivityIndicator *Activity;
@@ -178,7 +178,7 @@ public:
             (
                     parent,
                     // TRANSLATORS: %s is a cloud destination, e.g. "Crowdin" or ftp.wordpress.com etc.
-                    wxString::Format(_("Syncing with %s failed."), dest->GetName()),
+                    wxString::Format(_("Uploading translations to %s failed."), dest->GetName()),
                     _("Syncing error"),
                     wxOK | wxICON_ERROR
                 ));
