@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2015-2022 Vaclav Slavik
+ *  Copyright (C) 2015-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -30,14 +30,14 @@
 
 #include <memory>
 
-#include "concurrency.h"
+#include "cloud_accounts.h"
 #include "language.h"
 
 
 /**
     Client to the Crowdin platform.
  */
-class CrowdinClient
+class CrowdinClient : public CloudAccountClient
 {
 public:
     /// Return singleton instance of the client.
@@ -46,8 +46,11 @@ public:
     /// Destroys the singleton, must be called (only) on app shutdown.
     static void CleanUp();
 
+    static constexpr const char* SERVICE_NAME = "Crowdin";
+    const char *GetServiceName() const override { return SERVICE_NAME; }
+
     /// Is the user currently signed into Crowdin?
-    bool IsSignedIn() const;
+    bool IsSignedIn() const override;
 
     /// Wrap relative Crowdin URL to absolute URL with attribution
     static std::string AttributeLink(std::string page);
@@ -61,83 +64,64 @@ public:
      */
     dispatch::future<void> Authenticate();
     void HandleOAuthCallback(const std::string& uri);
-    bool IsOAuthCallback(const std::string& uri);
+    static bool IsOAuthCallback(const std::string& uri);
 
     /// Sign out of Crowdin, forget the token
-    void SignOut();
-
-    /// Information about logged-in user
-    struct UserInfo
-    {
-        std::wstring name;
-        std::wstring login;
-        std::string avatar;
-    };
+    void SignOut() override;
 
     /// Retrieve information about the current user asynchronously
-    dispatch::future<UserInfo> GetUserInfo();
-
-    /// Project listing info
-    struct ProjectListing
-    {
-        std::wstring name;
-        std::string identifier;
-        int id;
-    };
+    dispatch::future<UserInfo> GetUserInfo() override;
 
     /// Retrieve listing of projects accessible to the user
-    dispatch::future<std::vector<ProjectListing>> GetUserProjects();
+    dispatch::future<std::vector<ProjectInfo>> GetUserProjects() override;
 
-    /// File information
-    struct FileInfo
+    /// Retrieve details about given project
+    dispatch::future<ProjectDetails> GetProjectDetails(const ProjectInfo& project) override;
+
+    /// Create filename on local filesystem suitable for the remote file
+    std::wstring CreateLocalFilename(const ProjectInfo& project, const ProjectFile& file, const Language& lang) const override;
+
+    std::shared_ptr<FileSyncMetadata> ExtractSyncMetadata(Catalog& catalog) override;
+
+    /// Asynchronously download specific Crowdin file into @a output_file.
+    dispatch::future<void> DownloadFile(const std::wstring& output_file, const ProjectInfo& project, const ProjectFile& file, const Language& lang) override;
+
+    /// Asynchronously download specific Crowdin file into @a output_file.
+    dispatch::future<void> DownloadFile(const std::wstring& output_file, std::shared_ptr<FileSyncMetadata> meta) override;
+
+    /// Asynchronously upload specific Crowdin file data.
+    dispatch::future<void> UploadFile(const std::string& file_buffer, std::shared_ptr<FileSyncMetadata> meta) override;
+
+private:
+    class crowdin_http_client;
+    class crowdin_token;
+
+    struct FileInternal : public ProjectFile::Internal
     {
-        std::string title;
-        std::string fileName, dirName, branchName;
+        std::string fileName, dirName;
         std::string fullPath;
         int id, dirId, branchId;
     };
 
-    /// Project detailed information
-    struct ProjectInfo
+    struct CrowdinSyncMetadata : public FileSyncMetadata
     {
-        std::wstring name;
-        int id;
-        std::vector<Language> languages;
-        std::vector<FileInfo> files;
+        Language lang;
+        int projectId, fileId;
+        std::string xliffRemoteFilename;
+        std::string extension;
     };
 
-    /// Retrieve listing of projects accessible to the user
-    dispatch::future<ProjectInfo> GetProjectInfo(const int project_id);
-
-    /// Asynchronously download specific Crowdin file into @a output_file.
-    dispatch::future<void> DownloadFile(int project_id,
-                                        const Language& lang,
-                                        int file_id,
-                                        const std::string& file_extension,
-                                        bool forceExportAsXliff,
-                                        const std::wstring& output_file);
-
-    /// Asynchronously upload specific Crowdin file data.
-    dispatch::future<void> UploadFile(int project_id,
-                                      const Language& lang,
-                                      int file_id,
-                                      const std::string& file_extension,
-                                      const std::string& file_content);
-
-private:
     CrowdinClient();
     ~CrowdinClient();
 
     // Initialize m_api for use with given authorization; must be called before use
-    void InitWithAuthToken(const std::string& authToken);
+    bool InitWithAuthToken(const crowdin_token& token);
 
     void SignInIfAuthorized();
     void SaveAndSetToken(const std::string& token);
-    std::string GetValidToken() const;
+    crowdin_token GetValidToken() const;
 
-    class crowdin_http_client;
-
-    mutable std::unique_ptr<std::string> m_cachedAuthToken;
+    mutable std::unique_ptr<crowdin_token> m_cachedAuthToken;
     std::unique_ptr<crowdin_http_client> m_api;
     std::shared_ptr<dispatch::promise<void>> m_authCallback;
     std::string m_authCallbackExpectedState;
