@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2022 Vaclav Slavik
+ *  Copyright (C) 1999-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -59,13 +59,15 @@ class PoeditApp : public wxApp, public MenusManager
         /// Returns Poedit version string.
         wxString GetAppVersion() const;
         wxString GetAppBuildNumber() const;
-        bool IsBetaVersion() const;
         bool CheckForBetaUpdates() const;
 
-        // opens files in new frame
-        void OpenFiles(const wxArrayString& filenames, int lineno = 0);
+        // opens files in new frame, returns count of succesfully opened
+        int OpenFiles(const wxArrayString& filenames, int lineno = 0);
         // opens empty frame or catalogs manager
         void OpenNewFile();
+        // opens cloud translation with optional project to pre-open
+        template<typename T>
+        void OpenCloudTranslation(T preopen);
 
 #ifdef __WXOSX__
         void MacOpenFiles(const wxArrayString& names) override;
@@ -105,7 +107,7 @@ class PoeditApp : public wxApp, public MenusManager
         void OnNewFromScratch(wxCommandEvent& event);
         void OnNewFromPOT(wxCommandEvent& event);
         void OnOpen(wxCommandEvent& event);
-        void OnOpenFromCrowdin(wxCommandEvent& event);
+        void OnOpenCloudTranslation(wxCommandEvent& event);
         void OnOpenHist(wxCommandEvent& event);
         void OnAbout(wxCommandEvent& event);
         void OnWelcomeWindow(wxCommandEvent& event);
@@ -116,7 +118,6 @@ class PoeditApp : public wxApp, public MenusManager
         void OnGettextManual(wxCommandEvent& event);
 
 #ifdef __WXMSW__
-        void AssociateFileTypeIfNeeded();
         void OnWinsparkleCheck(wxCommandEvent& event);
         static int WinSparkle_CanShutdown();
         static void WinSparkle_Shutdown();
