@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2022 Vaclav Slavik
+ *  Copyright (C) 1999-2023 Vaclav Slavik
  *  Copyright (C) 2005 Olivier Sannier
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
@@ -682,10 +682,8 @@ void PoeditListCtrl::CreateColumns()
 #ifdef __WXOSX__
     NSTableView *tableView = (NSTableView*)[((NSScrollView*)GetHandle()) documentView];
     [tableView setIntercellSpacing:NSMakeSize(3.0, 0.0)];
-  #if __MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_16
     if (@available(macOS 11.0, *))
         tableView.style = NSTableViewStyleFullWidth;
-  #endif
 #endif
 
     m_colID = m_colIcon = m_colSource = m_colTrans = nullptr;
@@ -746,9 +744,23 @@ void PoeditListCtrl::UpdateColumns()
     auto srclang = m_catalog->GetSourceLanguage();
     auto lang = m_catalog->GetLanguage();
 
-    wxString sourceTitle = srclang.IsValid()
-                             ? wxString::Format(_(L"Source text — %s"), srclang.DisplayName())
-                             : _("Source text");
+    wxString sourceTitle;
+    if (m_catalog->UsesSymbolicIDsForSource())
+    {
+        // TRANSLATORS: Refers to symbolic ID of source text, i.e. when something like "button.label" is used instead of English text
+        sourceTitle = _("Source text ID");
+    }
+    else
+    {
+        sourceTitle = srclang.IsValid()
+                      ? wxString::Format(_(L"Source text — %s"), srclang.DisplayName())
+                      : _("Source text");
+    }
+    if (m_catalog->HasSideloadedReferenceFile())
+    {
+        wxFileName fn(m_catalog->GetSideloadedSourceData()->reference_file->GetFileName());
+        sourceTitle += wxString::Format(" [%s]", fn.GetFullName());
+    }
     m_colSource->SetTitle(sourceTitle);
 
 #ifdef __WXMSW__
@@ -821,10 +833,8 @@ void PoeditListCtrl::SizeColumns()
                 visibleCols++;
         }
         w -= 3 * visibleCols;
-  #if __MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_16
         if (@available(macOS 11.0, *))
             w -= 9;
-  #endif
     }
 #elif defined(__WXGTK__)
     w -= 2;
