@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2022 Vaclav Slavik
+ *  Copyright (C) 1999-2023 Vaclav Slavik
  *  Copyright (C) 2015 PrismJS (CSS parts)
  *  Copyright (c) 2013-2017 Cole Bemis (Feather Icons)
  *
@@ -132,11 +132,11 @@ FileViewer::FileViewer(wxWindow*)
 
     m_openInEditor = new wxButton(panel, wxID_ANY, MSW_OR_OTHER(_("Open in editor"), _("Open in Editor")));
 #ifdef __WXOSX__
-    static_cast<NSButton*>(m_openInEditor->GetHandle()).bezelStyle = NSRoundRectBezelStyle;
+    static_cast<NSButton*>(m_openInEditor->GetHandle()).bezelStyle = NSBezelStyleRoundRect;
 #endif
     m_topBarSizer->Add(m_openInEditor, wxSizerFlags().Center().ReserveSpaceEvenIfHidden().Border(wxLEFT, PX(10)));
 
-    sizer->Add(new wxStaticLine(panel, wxID_ANY, wxDefaultPosition, wxSize(-1, 1)), wxSizerFlags().Expand().Border(wxLEFT|wxRIGHT, PX(5)));
+    sizer->Add(new StaticLine(panel, wxID_ANY), wxSizerFlags().Expand().Border(wxLEFT|wxRIGHT, PX(5)));
 
 #ifdef __WXMSW__
     if (wxWebView::IsBackendAvailable(wxWebViewBackendEdge))
