@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2022 Vaclav Slavik
+ *  Copyright (C) 2013-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -267,6 +267,13 @@ void Language::Init(const std::string& code)
     {
         m_tag = DoGetLanguageTag(*this);
         m_icuLocale = m_tag;
+
+        char locale[512];
+        UErrorCode status = U_ZERO_ERROR;
+        uloc_forLanguageTag(m_tag.c_str(), locale, 512, NULL, &status);
+        if (U_SUCCESS(status))
+            m_icuLocale = locale;
+
         m_direction = DoIsRTL(*this) ? TextDirection::RTL : TextDirection::LTR;
     }
     else
@@ -314,8 +321,33 @@ std::string Language::Variant() const
         return m_code.substr(pos + 1);
 }
 
+Language Language::MinimizeSubtags() const
+{
+    if (m_icuLocale.empty())
+        return *this;
+
+    char minimized[512];
+    UErrorCode status = U_ZERO_ERROR;
+    uloc_minimizeSubtags(m_icuLocale.c_str(), minimized, 512, &status);
+    if (U_FAILURE(status))
+        return *this;
+
+    char tag[512];
+    uloc_toLanguageTag(minimized, tag, 512, /*strict=*/1, &status);
+    if (U_FAILURE(status))
+        return *this;
+
+    if (strcmp(tag, "zh") == 0)
+        return Language::FromLanguageTag("zh-Hans");
+
+    return Language::FromLanguageTag(tag);
+}
+
 Language Language::TryParse(const std::wstring& s)
 {
+    if (s.empty())
+        return Language(); // invalid
+
     if (IsValidCode(s))
         return Language(s);
 
@@ -375,6 +407,9 @@ Language Language::TryParseWithValidation(const std::wstring& s)
 
 Language Language::FromLanguageTag(const std::string& tag)
 {
+    if (tag.empty())
+        return Language(); // invalid
+
     char locale[512];
     UErrorCode status = U_ZERO_ERROR;
     uloc_forLanguageTag(tag.c_str(), locale, 512, NULL, &status);
@@ -515,8 +550,11 @@ const std::vector<std::wstring>& Language::AllFormattedNames()
 }
 
 
-Language Language::TryGuessFromFilename(const wxString& filename)
+Language Language::TryGuessFromFilename(const wxString& filename, wxString *wildcard)
 {
+    if (wildcard)
+        wildcard->clear();
+
     wxFileName fn(filename);
     fn.MakeAbsolute();
 
@@ -526,8 +564,15 @@ Language Language::TryGuessFromFilename(const wxString& filename)
     //  - directory name (cs_CZ, cs.lproj, cs/LC_MESSAGES)
     std::wstring name = fn.GetName().ToStdWstring();
     Language lang = Language::TryParseWithValidation(name);
-            if (lang.IsValid())
-                return lang;
+    if (lang.IsValid())
+    {
+        if (wildcard)
+        {
+            fn.SetName("*");
+            *wildcard = fn.GetFullPath();
+        }
+        return lang;
+    }
 
     size_t pos = name.find_first_of(L".-_");
     while (pos != wxString::npos)
@@ -535,24 +580,49 @@ Language Language::TryGuessFromFilename(const wxString& filename)
         auto part = name.substr(pos+1);
         lang = Language::TryParseWithValidation(part);
         if (lang.IsValid())
+        {
+            if (wildcard)
+            {
+                name.replace(pos+1, std::wstring::npos, L"*");
+                fn.SetName(name);
+                *wildcard = fn.GetFullPath();
+            }
             return lang;
-         pos = name.find_first_of(L".-_",  pos+1);
+        }
+        pos = name.find_first_of(L".-_",  pos+1);
     }
 
     auto dirs = fn.GetDirs();
     if (!dirs.empty())
     {
-        auto d = dirs.rbegin();
-        if (d->IsSameAs("LC_MESSAGES", /*caseSensitive=*/false))
+        size_t i = dirs.size() - 1;
+        if (dirs[i].IsSameAs("LC_MESSAGES", /*caseSensitive=*/false))
         {
-            if (++d == dirs.rend())
+            if (i == 0)
                 return Language(); // failed to match
+            --i;
+        }
+        wxString rest, wmatch;
+        if (dirs[i].EndsWith(".lproj", &rest))
+        {
+            lang = Language::TryParseWithValidation(rest.ToStdWstring());
+            wmatch = "*.lproj";
         }
-        wxString rest;
-        if (d->EndsWith(".lproj", &rest))
-            return Language::TryParseWithValidation(rest.ToStdWstring());
         else
-            return Language::TryParseWithValidation(d->ToStdWstring());
+        {
+            lang = Language::TryParseWithValidation(dirs[i].ToStdWstring());
+            wmatch = "*";
+        }
+        if (lang.IsValid())
+        {
+            if (wildcard)
+            {
+                fn.RemoveDir(i);
+                fn.InsertDir(i, wmatch);
+                *wildcard = fn.GetFullPath();
+            }
+            return lang;
+        }
     }
 
     return Language(); // failed to match
