@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2013-2022 Vaclav Slavik
+ *  Copyright (C) 2013-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -66,6 +66,9 @@ public:
     /// Return language tag for the language, per BCP 47, e.g. en-US or sr-Latn
     std::string LanguageTag() const { return m_tag; }
 
+    /// Minimizes the subtags, e.g. returns cs for cs-CZ, but en-GB for en-GB
+    Language MinimizeSubtags() const;
+
     /// Returns name of the locale suitable for ICU
     std::string IcuLocaleName() const { return m_icuLocale; }
     /// Returns ICU equivalent of the language info
@@ -155,8 +158,11 @@ public:
     /**
         Try to guess the language from filename, if the filename follows some
         commonly used naming pattern.
+
+        If @a wildcard is provided, sets it to a wildcard matching different language files
+        on success, or empty if language can't be guessed.
      */
-    static Language TryGuessFromFilename(const wxString& filename);
+    static Language TryGuessFromFilename(const wxString& filename, wxString *wildcard = nullptr);
 
     /**
         Try to detect the language from UTF-8 text.
@@ -167,6 +173,12 @@ public:
     static Language TryDetectFromText(const char *buffer, size_t len,
                                       Language probableLanguage = Language());
 
+    template<typename T>
+    static Language TryDetectFromText(const T& str, Language probableLanguage = Language())
+    {
+        return TryDetectFromText(str.data(), str.length(), probableLanguage);
+    }
+
     /// Returns object for English language
     static Language English() { return Language("en"); }
 
