@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2022 Vaclav Slavik
+ *  Copyright (C) 1999-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -151,11 +151,42 @@ int PreTranslateCatalog(wxWindow *window, CatalogPtr catalog, const PreTranslate
 
 void PreTranslateWithUI(wxWindow *window, PoeditListCtrl *list, CatalogPtr catalog, std::function<void()> onChangesMade)
 {
+    if (catalog->UsesSymbolicIDsForSource())
+    {
+        wxWindowPtr<wxMessageDialog> resultsDlg(
+            new wxMessageDialog
+                (
+                    window,
+                    _("Cannot pre-translate without source text."),
+                    _("Pre-translate"),
+                    wxOK | wxICON_ERROR
+                )
+        );
+        resultsDlg->SetExtendedMessage(_(L"Pre-translation requires that source text is available. It doesn’t work if only IDs without the actual text are used."));
+        resultsDlg->ShowWindowModalThenDo([resultsDlg](int){});
+        return;
+    }
+    else if (!catalog->GetSourceLanguage().IsValid())
+    {
+        wxWindowPtr<wxMessageDialog> resultsDlg(
+            new wxMessageDialog
+                (
+                    window,
+                    _("Cannot pre-translate from unknown language."),
+                    _("Pre-translate"),
+                    wxOK | wxICON_ERROR
+                )
+        );
+        resultsDlg->SetExtendedMessage(_(L"Pre-translation requires that source text’s language is known. Poedit couldn’t detect it in this file."));
+        resultsDlg->ShowWindowModalThenDo([resultsDlg](int){});
+        return;
+    }
+
     wxWindowPtr<wxDialog> dlg(new wxDialog(window, wxID_ANY, _("Pre-translate"), wxDefaultPosition, wxSize(PX(440), -1)));
     auto topsizer = new wxBoxSizer(wxVERTICAL);
     auto sizer = new wxBoxSizer(wxVERTICAL);
     auto onlyExact = new wxCheckBox(dlg.get(), wxID_ANY, _("Only fill in exact matches"));
-    auto onlyExactE = new ExplanationLabel(dlg.get(), _("By default, inaccurate results are filled in as well and marked as needing work. Check this option to only include accurate matches."));
+    auto onlyExactE = new ExplanationLabel(dlg.get(), _("By default, inaccurate results are also included, but marked as needing work. Check this option to only include perfect matches."));
     auto noFuzzy = new wxCheckBox(dlg.get(), wxID_ANY, _(L"Don’t mark exact matches as needing work"));
     auto noFuzzyE = new ExplanationLabel(dlg.get(), _("Only enable if you trust the quality of your TM. By default, all matches from the TM are marked as needing work and should be reviewed before use."));
 
