@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 2017-2022 Vaclav Slavik
+ *  Copyright (C) 2017-2023 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -70,7 +70,7 @@ public:
         if (!item->IsTranslated())
             return false;
 
-        auto syntax = SyntaxHighlighter::ForItem(*item, SyntaxHighlighter::Placeholder | SyntaxHighlighter::EnforceFormatTag);
+        auto syntax = SyntaxHighlighter::ForItem(*item, SyntaxHighlighter::Placeholder, SyntaxHighlighter::EnforceFormatTag);
         if (!syntax)
             return false;
 
